"""
This module contains the main-function that configures and starts batch runs.
"""

#Python built-in modules
# # # # # # # # # # # # #
import sys, os, time, re, math, pickle
from math import sqrt

#Project Modules
# # # # # # # # #

#Initialize OS-specific settings
import Ini
Ini.versionInit()

USE_PSYCO = True
"Attempts to use Psyco"
try:
	import psyco
	print "using Psyco"
except:
	print "Psyco not found"
	USE_PSYCO = False


#All the rest of the modules necessary to run tests are imported here
# # # 
import Classificators
import SearchUtils
import BC
import ScoreFunc
import BatchTests
#Functions for getting a single dataset
from Dataset import getRealTompa, getMchainTompa, getGenericTompa
from Dataset import getSynXMut, getSynFMut, getSynFMutZoops, getTransfacAlgMchain
from Dataset import getTransfacAlgReal, getTransfacModelReal, getTransfacModelMchain, getTransfacRemainingReal
from Dataset import getTransfacRemainingMchain
#Functions for getting all datasets of a specific type
from Dataset import getAllRealTompa, getAllMchainTompa, getAllGenericTompa
from Dataset import getAllSynXMut
from Dataset import getAllSynFMut, getAllSynFMutZoops, getAllTransfacAlgMchain, getAllTransfacAlgReal
from Dataset import getAllTransfacModelMchain, getAllTransfacModelReal, getAllTransfacRemainingReal
from Dataset import getAllTransfacRemainingMchain



def main():
	"""Runs training batch tests. Four lists as well as some additional variables
	describes what modules/variables should be used:
	
	B{models}: a list of the pattern model algorithms that should be used.
	
	B{scorefunctions}: a list of the scorefunctions to used with the pattern models.
	
	B{motiflengths}: a list of what motiflengths to search for. -1 gives the actual length of the motif
	
	B{datasets}: a list of what datasets to search in. Prefetched datasets can be gotten by
	I{getCREB, getRealTompa, getMchainTompa, getGenericTompa, getMotifsearch, getTransfac
	getAllCREB, getAllRealTompa, getAllMchainTompa, getAllGenericTompa, getAllMotifsearch, getAllTransfac}
	
	I{The total number of tests will be the number of models times the number of scorefunctions times 
	the number of motiflengths times the number of datasets.}
	
	B{writeMethod}: an integer that defines where to output results. The options originate from the L{BatchTests}
	module:
		- I{SCREEN}: outputs a summary of the results to standard output	
		- I{FILE}: writes results to file using the result-format described in the L{Result} module.
		- I{FORCE_RUN}: by default, a test is not run if a result file for a similar test exists. Enforcing test runs is done with this parameter. 
		- The options can be combined: SCREEN | FORCE_RUN
	
	B{hdistSpecific}: Hamming distance algorithms only: A tuple containing the minimum and maximum Hamming distance 
	value to search for.
	
	B{regSpecific}: IUPAC algorithms only. Float value giving an input score to an IUPAC algorithm, only useful with
	branch and bound approaches. If set to I{None}, if a result file exists for a similar test with a lower pattern length,
	the score from that result will be used.
	
	
	The test is started by initializing a batchtest class from L{BatchTests} with the given parameters,
	and invoking the method I{runAll()}.
	
	
	
	"""
		
		
	
	
	
	#Use the specialized compiler psyco. Comment it out when running time benchmarks.
	#--------------------------------------------------------------------------------
	if USE_PSYCO: psyco.full()
		
	
	#Scoring functions that can be used in tests
	#-------------------------------------------
	#:Correlation coefficient
	c1 = ScoreFunc.wcc(0)
	#:fw-measure with weigth w
	c2 = ScoreFunc.fw(w=1)
	#:Accurracy with weigth w. w=0 turns off weigths
	c3 = ScoreFunc.accuracy(w=1)
	
	#Model instances that can be used in tests
	#Usually, 	m_hdi is used for Hamming distance patterns, 
	#			m_reg is used for IUPAC patterns,
	#and		m_pwm is used for PWM patterns
	#-----------------------------------------
	#HAMMING DISTANCE
	# # # # # # # # #
	#:Default Hamming distance class
	m_hdi = Classificators.Mismatch() #if available, it is set to Mismatch_cmodule, elsewise Mismatch_basic
	#:Basic Hamming distance class
	m_hdb = Classificators.Mismatch_basic()
	#:Hamming distance algorithm using a c++ Hamming distance library (not always available)
	m_hdc = Classificators.Mismatch_cmodule()
	#:Hamming distance algorithm using a q-gram. Somewhat faster than basic for small lengths/distances
	m_hdq = Classificators.Mismatch_qgram()
	
	#Benchmark models
	m_hdt	= Classificators.Mismatch_tree()
	m_hdbl	= Classificators.Mismatch_lessexhaustive()
	m_hde	= Classificators.Mismatch_exhaustive()
	m_hdqf	= Classificators.Mismatch_qgram_filter()
	
	#IUPAC
	# # # #
	#:Branch 'n bound-approach, all optimizations on. 
	m_reg = Classificators.Regexp()
	
	#Benchmark models
	m_ren = Classificators.Regexp_basic()
	m_reb = Classificators.Regexp_branch()
	m_rec = Classificators.Regexp_combos()
	m_re2 = Classificators.Regexp_combos2()
	
	#POSITION WEIGTH MATRIX
	# # # # # # # # # # # #
	#:Basic PWM
	m_pwm = Classificators.PWM()
	#:New, expanded PWM model
	m_pwn = Classificators.PWM_new()
	#:Higher order background model
	m_pbm = Classificators.PWM_BM_new()
	
	
	#OPTIMAL MODEL
	# # # # # # # #
	#:Iterates the possible variations of TP- and FP-values.
	m_opt = Classificators.Optimal()
	#:Faster approach, looks at ratios between positive and negative set.
	m_opn = Classificators.Optimal_new()
	
	# # # # # # # # # # # # # # # # # #
	#
	#PARAMETERS TO USE IN THE TEST
	# # # # # # # # # # # # # # # # # #
	models = [m_hdi]
	scorefunctions = [c1]
	motiflengths = [-1]
	writeMethod =  BatchTests.SCREEN# | BatchTests.FILE# | BatchTests.FORCE_RUN
	datasets = getAllTransfacAlgMchain()
	if type(datasets) != type([]): datasets = [datasets]

	#Set parameters specific for each model
	#--------------------------------------
	
	#Hamming distance model
	mindist = 1
	maxdist = 2
	hdistSpecific = [mindist, maxdist] #The minimum and maximum number of mismatches
	#IUPAC model
	regSpecific = -1. #Manual input of the best found score for the IUPAC algorithm
	
	
	#Create batch test instance and run test(s)
	#
	#A batch test is run by initializing a given Batch test class with the given parameters
	#and calling runAll()
	#------------------------------------------
	bt = BatchTests.Basic(models, datasets, scorefunctions, motiflengths,\
			writeMethod=writeMethod, hdistSpecific=hdistSpecific, regSpecific = regSpecific)
	
	#bt = BatchTests.Blocks(models, datasets, scorefunctions, motiflengths,\
	#		writeMethod=writeMethod, hdistSpecific=hdistSpecific, regSpecific = regSpecific)
	
	#bt = BatchTests.CVSBlocks(models, datasets, scorefunctions, motiflengths,\
	#		writeMethod=writeMethod, hdistSpecific=hdistSpecific, regSpecific = regSpecific, excluded=[], cvnum = 1)

	
	#bt = BatchTests.CVSBlocksTest(models, datasets, scorefunctions, motiflengths,\
	#	writeMethod=writeMethod, hdistSpecific=hdistSpecific, regSpecific = regSpecific, excluded=[], cvnum = 1)
	
	#bt = BatchTests.CVSBasicTest(models, datasets, scorefunctions, motiflengths,\
	#		writeMethod=writeMethod, hdistSpecific=hdistSpecific, regSpecific = regSpecific, excluded=[], cvnum = 1)
	#bt = BatchTests.CVSBasic(models, datasets, scorefunctions, motiflengths,\
	#		writeMethod=writeMethod, hdistSpecific=hdistSpecific, regSpecific = regSpecific, excluded=[], cvnum = 1)
	bt.runAll()
	
	
if __name__ == '__main__':
	main()
